﻿using IduIT.cardo.Core.Api.Applications.Utils;
using IduIT.Core.PropertyModel;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace $safeprojectname$.Application
{
	/*
	TODO: Einstellungen anpassen
	Die Vorlage enthält häufig verwendetet Einstellungen, hier Datenbank + Schema und ein EpsgCode für Geometrie-Daten
	*/
	/// <summary>
	/// Die Konfiguration der Anwendung
	/// </summary>
	internal sealed class ApplicationConfiguration : IduIT.cardo.Core.Api.Applications.CardoManagedApplicationSettings,
		cardo.Core.Api.Applications.Utils.CardoManagedApplicationSettingsUtil.IDatabaseSettings
	{
		public static IduIT.Core.PropertyModel.Auto.LayoutType LAYOUT(string typeName)
		{
			return AssemblySourceInfo.GetPropertyModelLayout(typeName);
		}

		///////////////////////////////////////////////////
		#region Einstellungen - Datenbankverbindung
		///////////////////////////////////////////////////
		/// <summary>
		/// Datenbank - Connection
		/// </summary>
		[PropertyModelProperty]
		public String DbConnectionString
		{
			get;
			set;
		}
		/// <summary>
		/// Schema-Name
		/// </summary>
		[PropertyModelProperty]
		public String DbSchemaName
		{
			get;
			set;
		}

		/// <summary>
		/// Projektion der Geometrien.
		/// </summary>
		[PropertyModelProperty(ValueHandlerType = typeof(IduIT.cardo.Core.PropertyModel.ValueHandler.AvailableProjection))]
		public IDU.GeoLib.Epsg DataEpsg
		{
			get;
			set;
		}
		///////////////////////////////////////////////////
		#endregion
		///////////////////////////////////////////////////
		#region Öffentliche Methoden
		///////////////////////////////////////////////////
		/// <summary>
		/// Öffnet die Verbindung zur Datenbank und initialisiert ggf. die Einstellungen
		/// </summary>
		/// <returns></returns>
		public IDU.DB.IDataBaseHandler OpenDatabase()
		{
			return CardoManagedApplicationSettingsUtil.OpenDatabaseConnection(this, true);
		}

		///////////////////////////////////////////////////
		#endregion
		///////////////////////////////////////////////////
		#region Basis-Klasse CardoManagedApplicationSettings 
		///////////////////////////////////////////////////
		/// <summary>
		/// Valdierung der Einstellungen
		/// </summary>
		/// <returns></returns>
		public override IduIT.Core.PropertyModel.Validation.ValidationResult Validate()
		{
			//Das Ergebnis ...
			var r = new IduIT.Core.PropertyModel.Validation.ValidationResult();

			//DB-Einstellungen
			CardoManagedApplicationSettingsUtil.ValidateAllDatabaseSettings(r,
					this,
					schemaBehavior: CardoManagedApplicationSettingsUtil.DbSchemaBehavior.MustSpecifyForPostgreSQL,
					minVersions: new[] {
						Tuple.Create(IDU.DB.WellKnownDatabaseEngines.PostgreSqlServer, new Version(10, 0)),
						Tuple.Create(IDU.DB.WellKnownDatabaseEngines.MsSqlServer, new Version(0, 0)),
					}
				);

			//EPSG
			if (DataEpsg == null)
				r.AddMessage(() => DataEpsg, "Die Projektion der Geometrien wurde nicht angegeben.");
			else if (DataEpsg.IsUnknown || DataEpsg.IsVerticalCs)
				r.AddMessage(() => DataEpsg, "Die Projektion ist ungültig oder 1-dimensional.");



			return r;
		}
		///////////////////////////////////////////////////
		#endregion
		///////////////////////////////////////////////////
		#region Privates 
		///////////////////////////////////////////////////
	
		///////////////////////////////////////////////////
		#endregion
		///////////////////////////////////////////////////
	}


}
