﻿using System.Linq;
using IDU.Core.Web.Res;
using $safeprojectname$.Extensions;
using IduIT.cardo.Core.Api.Applications.SqlUpdate;
using System.Collections.Generic;
namespace $safeprojectname$.Application
{
	/// <summary>
	/// Anwendungsvorlage
	/// </summary>
	[IduIT.Core.Web.Ajax.AjaxClass("$safeprojectname$.Cardo4AnwendungRemote")]
	internal sealed class Cardo4Anwendung : IduIT.cardo.Core.Api.Applications.ICardoManagedApplicationWithSystemSettings<ApplicationConfiguration>,
		 IduIT.cardo.Core.Applications.IStandaloneClientApplicationDefinition,
		 IduIT.cardo.Core.Api.Applications.ICardoManagedApplicationWithSQLUpdateSupport
	{
		//TODO: Evtl. nicht benötigte Schnittstellen und Methoden vor dem 1. Registrieren der Anwendung löschen.

		////////////////////////////////////////////////////////////////////
		#region Cardo-Anwendungsdetails, aus der Projektvorlage
		////////////////////////////////////////////////////////////////////
		////////////////////////////////////////////////////////////////////
		#region ICardoManagedApplication (Teils IClientApplicationDefinition)
		////////////////////////////////////////////////////////////////////

		public string InitOnlyTitle
		{
			get
			{
				return "Titel der Anwendung";
			}
		}
		public string InitOnlyDescription
		{
			get
			{
				return "Beschreibung der Anwendung";
			}
		}

		public void OnLoadRegisteredApplication(string registeredInstanceId)
		{
			_currentInstanceId = registeredInstanceId;
		}
		/// <summary>
		/// Wie der Name schon sagt.
		/// </summary>
		/// <returns></returns>
		public bool InitOnlyIsVisibleInThisInstallation()
		{
			return true;
		}
		public bool InitOnlyAllowMultipleRegistration()
		{
			return false;
		}
		public void Dispose()
		{
			_orm = null;
			_db?.Dispose();
			_db = null;
		}
		////////////////////////////////////////////////////////////////////
		#endregion
		////////////////////////////////////////////////////////////////////
		#region IClientApplicationDefinition
		////////////////////////////////////////////////////////////////////
		/// <summary>
		/// CSS-Klasse für das Icon der Anwendung in der Oberfläche. Es muss dafür gesorgt werden,
		/// dass in RegisterDefnitionScripts das CSS für das Icon registriert wird.
		/// </summary>
		public string IconCls
		{
			get
			{
				//die -fa Icons entsprechen fontAwesome
				//siehe http://fontawesome.io/icons/
				//i.d.R. sollte eine eigene Icon-Klasse erstellt werden

				return "iduit-core-icon-fa iduit-core-icon-fa-share-alt";
			}
		}
		/// <summary>
		/// sind mehrfache Instanzen möglich?
		/// </summary>
		public bool AllowMultiple
		{
			get
			{
				return false;
			}
		}
		/// <summary>
		/// Registriert Scripte in der Seite, die für die Anwendungsinstanz benötigt werden.
		/// </summary>
		public void RegisterApplicationScripts(ResourceCollector resourceCollector)
		{
			//TODO: ggf. den Namen der Script-Datei anpassen
			resourceCollector.AddScriptsFromSourceExecutingAssembly(
				"TypeScript/Output/$safeprojectname$.Application.Components.js");
		}

		/// <summary>
		/// Registriert Scripte in der Seite, die ggf. für die Anwendungsdefinition verfügbar sein müssen.
		/// </summary>
		public void RegisterDefinitionScripts(ResourceCollector resourceCollector)
		{
			IduIT.Core.Web.Icons.Registration.RegisterFontAwesomeInPage(resourceCollector);

			//TODO: ggf. den Namen der Css-Datei anpassen
			resourceCollector.AddStyleSheetsFromSourceExecutingAssembly(
				"Application/Resources/css/Application.css");

			//TODO: ggf. den Namen der Script-Datei anpassen
			resourceCollector.AddScriptsFromSourceExecutingAssembly(
				"TypeScript/Output/$safeprojectname$.Application.js");

		}

		/// <summary>
		/// Bietet die Möglichkeit Wortfragmente zurückzugeben, die, wenn so in der Schnellstartleiste eingegeben,
		/// die Anwendung als Treffer liefern.
		/// </summary>
		/// <returns></returns>
		public IEnumerable<string> GetSearchWords()
		{
			yield return "DIE Anwendung";
		}

		////////////////////////////////////////////////////////////////////
		#endregion
		////////////////////////////////////////////////////////////////////
		#region ICardoManagedApplicationWithSystemSettings
		////////////////////////////////////////////////////////////////////
		/// <summary>
		/// Die Einstellungen, werden von cardo bei der Erstellung der Instanz immer mit den eingestellten Werten belegt.
		/// </summary>
		public ApplicationConfiguration Settings
		{
			get;
			set;
		}

		public IduIT.Core.PropertyModel.Validation.ValidationResult ValidateGivenSettings(ApplicationConfiguration settings)
		{
			//Einstellungen validieren
			var result = settings.Validate();
			//was anderes scheint hier in der Anwendung nicht sinnvoll zu sein
			return result;
		}


		////////////////////////////////////////////////////////////////////
		#endregion
		////////////////////////////////////////////////////////////////////
		#region ICardoManagedApplicationWithSQLUpdateSupport
		////////////////////////////////////////////////////////////////////
		/// <summary>
		/// Die Anwendung muss hier ein Beschreibungsobjekt für das SQL-Update zurückgeben. 
		/// In dem Objekt wird eine geöffnete Datenbank erwartet, in der keine Transaktion vorhanden sind.
		/// Die Datenbankverbindung wird garantiert vom Aufrufer wieder geschlossen.
		/// </summary>
		/// <returns></returns>
		public IEnumerable<ApplicationSqlUpdateDescription> GetSqlUpdateDescriptionInstances()
		{
			//einfach die Datei aus der Resource rausgeben
			System.Xml.XmlDocument sqlFile = new System.Xml.XmlDocument();

#if DEBUG
			sqlFile.Load(AssemblySourceInfo.SourceCodeBaseFolder + @"\Application\SqlUpdate.xml");
#else
			sqlFile.LoadXml(Properties.Resources.SqlUpdate);
#endif

			yield return new ApplicationSqlUpdateDescription(
				this.Settings.OpenDatabase(),
				sqlFile);
		}

		////////////////////////////////////////////////////////////////////
		#endregion
		////////////////////////////////////////////////////////////////////
		#region Berechtigungen
		////////////////////////////////////////////////////////////////////
		/// <summary>
		/// Berechtigungen des aktuellen Benutzers.
		/// Es ist garantiert, dass nie null zurückgegeben wird. 
		/// Wird von cardo immer mit den eingestellten Werten zum aktuellen Benutzerkontext belegt.
		/// Die Eigenschaft kann public oder internal sein.
		/// </summary>
		[System.ComponentModel.Description("Berechtigungen")]
		internal ApplicationRights Permission { get; } = new ApplicationRights();
		////////////////////////////////////////////////////////////////////
		#endregion
		////////////////////////////////////////////////////////////////////
		#region Private Methoden und Member
		////////////////////////////////////////////////////////////////////

		/// <summary>
		/// Zugriff auf die Datenbank.
		/// Die Datenbank wird im Dispose der Anwendung automatisch wieder geschlossen.
		/// </summary>
		private IDU.DB.IDataBaseHandler Database
		{
			get
			{
				if (_db == null)
				{
					_db = this.Settings.OpenDatabase();
				}
				return _db;
			}
		}

		/// <summary>
		/// Zugriff auf den Objekt-Mapper zur Datenbank <see cref="Database"/>
		/// </summary>
		internal IduIT.Core.Data.ORM.OrmHandler Orm
		{
			get
			{
				if (null == _orm)
					_orm = new IduIT.Core.Data.ORM.OrmHandler(this.Database);
				return _orm;
			}
		}

		private string _currentInstanceId;
		private IDU.DB.IDataBaseHandler _db;
		private IduIT.Core.Data.ORM.OrmHandler _orm;

		////////////////////////////////////////////////////////////////////
		#endregion
		////////////////////////////////////////////////////////////////////
		#endregion
		////////////////////////////////////////////////////////////////////

		////////////////////////////////////////////////////////////////////
		#region Eigene Methoden
		////////////////////////////////////////////////////////////////////

		//TODO: Implementierung der Anwendungs-Logik

		/*
		Alle Ajax-Methoden müssen in dieser Klasse ausgeführt werden, wenn der Zugriff auf Einstellungen
		oder Berechtigungen der Anwendung erforderlich ist.
		Es sollte die IduDeveloper-Extension für VisualStudio installiert werden. Rufen Sie im Kontextmenü der Anwendung
		nach Anpassen der Ajax-Methoden oder deren Argumenten den Befehl "IduIT: Create .NET TypeScript Code" auf.

		Die Methode wird danach als $safeprojectname$.Cardo4AnwendungRemote.AxEcho(string) : promise<String>)
		verfügbar sein:

		[IduIT.Core.Web.Ajax.AjaxMethod]
		public string AxEcho(string msg)
		{
			return msg;
		}
		*/
		////////////////////////////////////////////////////////////////////
		#endregion
		////////////////////////////////////////////////////////////////////
	}
}
