﻿namespace $safeprojectname$.Application
{
	/**
	 * Anwendungsklasse.
	 * Wichtig: diese Datei sollte die einzige sein, welche bei der Registrierung der Anwendung raus geschrieben wird (RegisterDefinitionScripts).
	 * Die andere Komponenten werden nachgeladen (siehe RegisterApplicationScripts).
	 */
	export class Cardo4Anwendung extends IduIT.Cardo4.Apps.ApplicationBase implements
		// Support für Sitzungs-Status
		IduIT.cardo.Core.CoreModules.Personalization.Session.ISessionStateful<void>
	{
		/**
		 * Links zur Anwendungshilfe. Diese werden im Anwendungsfenster verlinkt.
		 */
		public static readonly helpLinks: IduIT.Cardo4.Apps.ApplicationHelp.IApplicationHelpConfig = {
			main: '?pgId=0'
		}

		/**
		 * Statische Funktion, die prüft, ob die Anwendung mit der spezifischen übergebenen WellKnownType-Instanz umgehen kann.
		 * Dies kann bspw. bei fehlenden Berechtigungen nicht der Fall sein.
		 * Kann optional spezielle Optionen zum Starten der Anwendung mit dem Typ zurückgeben.
		 * 
		 * @param wkt Eine WellKnownType-Instanz.
		 */
		public static canHandleWellKnownType(wkt: IduIT.Cardo4.Wkt.IWellKnownType): IduIT.Cardo4.Apps.TcanHandleWellKnownTypeUnion
		{
			// Beispiel: Die Anwendung kann eine Ebene aus dem Admin-Tree verarbeiten.
			/*
				return wkt instanceof IduIT.Cardo4.Wkt.SourceId;
			*/

			// TODO: entsprechende Implementierung
			return false;
		}


		/**
		 * Erstellt eine Instanz der Anwendung.
		 * @internal
		 */
		public constructor()
		{
			super();

			this.__mainComponent = new Components.MainComponent();
		}

		/**
		 * Gibt die UI-Komponenten der Anwendung zurück.
		 * 
		 * Diese Methode wird 1x pro Anwendung aufgerufen.
		 * Alle Komponente, die in dieser Methode erstellt werden, können als Referenz (this.xxx) in der Anwendung abgelegt werden
		 * und bleiben bestehen, solange die Anwendungsinstanz existiert.
		 * 
		 * Wie diese Komponenten arrangiert werden, bleibt allerdings dem Layout der Umgebung überlassen.
		 */
		public getUi(): IduIT.Cardo4.Apps.IApplicationUi | null
		{
			return {
				component: this.__mainComponent,
				width: 700,
				height: 400
			};
		}

		/**
		 * Startet die Anwendung mit dem angegebenen WellKnownType.
		 * 
		 * Für diesen Typ wurde in canHandleWellKnownType true zurückgegeben.
		 * 
		 * @param wkt WellKnownType-Instanz.
		 * @param option Eine Option, die zum Starten ausgewählt wurde. Optionen können über die statische Methode
		 * canHandleWellKnownType zurückgegeben werden. Ist dieser Parameter nicht angegeben, wurde keine Option
		 * ausgewählt und es ist die Standard-Aktion auszuführen.
		 */
		public handleWellKnownType(wkt: IduIT.Cardo4.Wkt.IWellKnownType, option?: IduIT.Cardo4.Apps.StartOptions.IStartOptionEntry<string>): IduIT.Promise.IPromise<void>
		{
			// TODO: Implementierung für alle Möglichkeiten, die in canHandleWellKnownType potentiell true bzw. eine Liste von Optionen zurückgeben.
			/*
			if (wkt instanceof IduIT.Cardo4.Wkt.SourceId)
			{

			}
			*/
			return IduIT.Promise.Deferred.resolve(undefined);
		}

		/**
		 * Prüft, ob noch ungespeicherte Daten vorhanden sind, bevor die Anwendung geschlossen wird.
		 * (Überladung von ApplicationBase)
		 */
		public beforeCloseHandler(continueClose: TVoidFunction): boolean
		{
			if (this.__mainComponent.hasUnsavedChanges)
			{
				// TODO: Implementierung ...
				return false;
			}
			else
			{
				return true;
			}
		}

		/**
		 * Wird beim Schließen einer Anwendung bzw. von cardo aufgerufen und
		 * gibt ein Objekt zurück, mit dem die Klasse in der Lage ist, über restoreSessionState
		 * ihren Zustand wieder herzustellen.
		 */
		public getSessionState(): void
		{
			// Implementierung (vorerst) nur, damit gespeichert wird, dass die Anwendung wieder mit geöffnet wird.
			return;
		}

		/**
		 * Wird aufgerufen, wenn ein Sitzungszustand bekannt ist und wieder hergestellt werden soll.
		 * Das Promise muss aufgelöst werden, wenn die Aktion beendet ist.
		 * 
		 * @param state Der gespeicherte Sitzungszustand.
		 */
		public restoreSessionState(state: void): IduIT.Promise.IPromise<void>
		{
			return IduIT.Promise.Deferred.resolve<void>(undefined);
		}

		/**
		 * Disposing.
		 */
		public dispose(): void
		{
			super.dispose();
			Kiss.Util.Disposer.disposeMembers(this, '__mainComponent');
		}

		/**
		 * Komponente für die Bearbeitung der Einstellungen.
		 */
		private readonly __mainComponent: Components.MainComponent;
	}
}
