﻿using IDU.Core.Web.Res;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;


namespace $safeprojectname$.Extensions
{
	/// <summary>
	/// Erweiterungsmethoden für den ResourceCollector
	/// </summary>
	internal static class ResourceCollectorExtensions
	{
		/// <summary>
		/// Fügt den Assembly-Namespace an die Ressourcen-Pfade an, falls noch nicht vorhanden
		/// </summary>
		/// <param name="resourceNames"></param>
		/// <returns></returns>
		private static string[] ApplyAssemblyNamespaceToPath(string[] resourceNames)
		{
			if (resourceNames == null)
				return null;
			for (int i = 0; i < resourceNames.Length; i++)
				resourceNames[i] = ApplyAssemblyNamespaceToPath(resourceNames[i]);
			return resourceNames;
		}
		/// <summary>
		/// Fügt den Assembly-Namespace an einen Ressourcen-Pfad an, falls noch nicht vorhanden
		/// </summary>
		/// <param name="resourceName"></param>
		/// <returns></returns>
		private static string ApplyAssemblyNamespaceToPath(string resourceName)
		{
			if (resourceName == null)
				return null;
			if (!resourceName.StartsWith(AssemblySourceInfo.DefaultNamespace))
				resourceName = AssemblySourceInfo.DefaultNamespace + "/" + resourceName.TrimStart('/', '\\');
			return resourceName;
		}

		/// <summary>
		/// Fügt ein oder mehrere Scripts aus der ausführenden Assembly hinzu.
		/// </summary>
		/// <param name="rc">ResourceCollector</param>
		/// <param name="resourcePathsInLib">Die Ressourcen-Pfade</param>
		public static void AddScriptsFromSourceExecutingAssembly(this ResourceCollector rc, params string[] resourcePathsInLib)
		{
			rc.AddScriptsFromResource(
				System.Reflection.Assembly.GetExecutingAssembly(),
				AssemblySourceInfo.SourceCodeBaseFolder,
				ApplyAssemblyNamespaceToPath(resourcePathsInLib));
		}

		/// <summary>
		/// Registriert ein oder mehrere Stylesheets  aus der ausführenden Assembly hinzu.
		/// </summary>
		/// <param name="rc">ResourceCollector</param>
		/// <param name="resourcePathsInLib">Die Ressourcen-Pfade</param>
		public static void AddStyleSheetsFromSourceExecutingAssembly(this ResourceCollector rc, params string[] resourcePathsInLib)
		{
			rc.AddStyleSheetsFromResource(
				System.Reflection.Assembly.GetExecutingAssembly(),
				AssemblySourceInfo.SourceCodeBaseFolder,
				ApplyAssemblyNamespaceToPath(resourcePathsInLib));
		}

	}
}
